meta:
  id: frame_tlm_entry_sat
  title: EntrySat TLM Frame
  file-extension: .data
  license:  LGPL V2
  ks-version: 0.8
  endian: be

  

seq:
  - id: frame_tlm_entry_sat
    type: ax25_frame
    doc-ref: 'https://www.tapr.org/pub_ax25.html'

types:
  ax25_frame:
    seq:
    - id: ax25_header
      type: ax25_header
    - id: payload
      type:
        switch-on: ax25_header.ctl & 0x13
        cases:
          0x03: ui_frame_tlm_entry_sat
          0x13: i_frame
          0x00: i_frame
          0x02: i_frame
          0x10: i_frame
          0x12: i_frame
          #0x11: s_frame

  ax25_header:
    seq:
      - id: dest_callsign_raw
        type: dest_callsign_raw
      - id: dest_ssid_raw
        type: u1
      - id: src_callsign_raw
        type: src_callsign_raw
      - id: src_ssid_raw
        type: u1
      - id: ctl
        type: u1
    instances:
      src_ssid:
        value: (src_ssid_raw & 0x0f) >> 1
      dest_ssid:
        value: (dest_ssid_raw & 0x0f) >> 1
  dest_callsign_raw:
    seq:
      - id: dest_callsign_ror
        process: ror(1)
        size: 6
        type: dest_callsign
  src_callsign_raw:
    seq:
      - id: src_callsign_ror
        process: ror(1)
        type: src_callsign
        size: 6
  dest_callsign:
    seq:
      - id: dest_callsign
        type: str
        encoding: ASCII
        size: 6
  src_callsign:
    seq:
      - id: src_callsign
        type: str
        encoding: ASCII
        size: 6

  i_frame:
    seq:
      - id: pid
        type: u1
      - id: ax25_info
        size-eos: true

  ui_frame_tlm_entry_sat:
    seq:
      - id: pid
        type: u1
      - id: time 
        type: u4
      - id: packetheader
        type: packetheader
      - id: telemetry_data_filed_hearder
        type: telemetry_data_filed_hearder
      - id: telemetry_data
        type: telemetry_data
      - id: ax25_info
        size-eos: true
        
  packetheader:
    seq:
      - id: version
        type: b3
      - id: packet_type
        type: b1
      - id: packet_flag
        type: b1
      - id: packet_apid
        type: b11	
      - id: sequence_control
        type: u2
      - id: packet_lenght
        type: u2      
        
  telemetry_data_filed_hearder:
    seq:
      - id: spare1
        type: b1	  
      - id: pus_version
        type: b3
      - id: spare2
        type: b4
      - id: service
        type: u1
      - id: service_subtype
        type: u1
      - id: time2 
        size: 5
  telemetry_data:
    seq:
      - id: sid
        type: u1
      - id: mode_safe
        type: u1
        doc:  Operational status.
         0 = Safe mode
         1 = Orbital or Entry mode
      - id: eps_vbatt
        type: u1
        doc: Battery voltage in V
         formulae 𝑦 = 3 + 0.05×𝐊      
      - id: eps_batt_vcurrent
        type: u1
        doc: Battery bus current in A
         formulae  𝑦 = −1 + 0.0078740×𝑥
      - id: eps_3_3_v_current
        type: u1
        doc: 3.3V bus current in A
         formulae  𝑦 = 0.025*x
      - id: eps_5_v_current
        type: u1
        doc: 5V bus current in A
         formulae  𝑦 = 0.025*x
      - id: trx_temp
        type: u1
        doc: Transceiver’s temperature in °C
         formulae  𝑦 = −15 + 0.25×𝑥
      - id: eps_temp
        type: u1
        doc: Power system’s temperature °C
         formulae  𝑦 = −15 + 0.25×𝑥
      - id: batt_temp
        type: u1
        doc: Batteries temperature in °C
         formulae  𝑦 = −15 + 0.25×𝑥
        